@@ -1,4 +1,4 @@
-#!/opt/local/bin/perl
+#!/usr/bin/perl
 use utf8;
 binmode stdin,":utf8";
 binmode stdout,":utf8";
@@ -48,7 +48,7 @@ s/d-h/ده/g;
 s/A/ا/g;
 s/b/ب/g;
 s/t\*/ة/g;
-s/th/ث/g;
+s/c/ث/g;
 s/t/ت/g;
 s/j/ج/g;
 s/H/ح/g;
