@@ -1,6 +1,6 @@
-# $Id: TLWinGoo.pm 53076 2019-12-10 06:20:44Z preining $
+# $Id: TLWinGoo.pm 63068 2022-04-18 05:58:07Z preining $
 # TeXLive::TLWinGoo.pm - Windows goop.
-# Copyright 2008-2018 Siep Kroonenberg, Norbert Preining
+# Copyright 2008-2021 Siep Kroonenberg, Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
@@ -9,9 +9,11 @@
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
+#use strict; use warnings; notyet
+
 package TeXLive::TLWinGoo;
 
-my $svnrev = '$Revision: 53076 $';
+my $svnrev = '$Revision: 63068 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -24,7 +26,7 @@ sub module_revision { return $_modulerevision; }
 
 =head1 NAME
 
-C<TeXLive::TLWinGoo> -- Additional utilities for Windows
+C<TeXLive::TLWinGoo> -- TeX Live Windows-specific support
 
 =head2 SYNOPSIS
 
@@ -32,8 +34,6 @@ C<TeXLive::TLWinGoo> -- Additional utilities for Windows
 
 =head2 DIAGNOSTICS
 
-  TeXLive::TLWinGoo::is_vista;
-  TeXLive::TLWinGoo::is_seven;
   TeXLive::TLWinGoo::is_ten;
   TeXLive::TLWinGoo::admin;
   TeXLive::TLWinGoo::non_admin;
@@ -80,7 +80,7 @@ C<TeXLive::TLWinGoo> -- Additional utilities for Windows
 
 All exported functions return forward slashes.
 
-=head2 DESCRIPTION
+=head1 DESCRIPTION
 
 =over 4
 
@@ -88,11 +88,9 @@ All exported functions return forward slashes.
 
 BEGIN {
   use Exporter;
-  use vars qw( @ISA @EXPORT $Registry);
+  use vars qw( @ISA @EXPORT @EXPORT_OK $Registry);
   @ISA = qw( Exporter );
   @EXPORT = qw(
-    &is_vista
-    &is_seven
     &is_ten
     &admin
     &non_admin
@@ -140,7 +138,6 @@ BEGIN {
     $Registry->Delimiter('/');
     $Registry->ArrayValues(0);
     $Registry->FixSzNulls(1);
-    require Win32::OLE::NLS;
     require Win32::Shortcut;
     Win32::Shortcut->import( qw( SW_SHOWNORMAL SW_SHOWMINNOACTIVE ) );
     require Time::HiRes;
@@ -162,9 +159,7 @@ sub reg_debug {
 
 my $is_win = ($^O =~ /^MSWin/i);
 
-# Win32: import some wrappers for API functions
-# failed to get Win32::API::More to work with raw API functions;
-# for now, use a tiny dll with perl-friendly wrappers
+# Win32: import wrappers for some horrible API functions
 
 # import failures return a null result;
 # call imported functions only if true/non-null
@@ -172,9 +167,9 @@ my $is_win = ($^O =~ /^MSWin/i);
 my $SendMessage = 0;
 my $update_fu = 0;
 if ($is_win) {
-  $SendMessage = new Win32::API('user32', 'SendMessageTimeout', 'LLPPLLP', 'L');
+  $SendMessage = Win32::API::More->new('user32', 'SendMessageTimeout', 'LLPPLLP', 'L');
   debug ("Import failure SendMessage\n") unless $SendMessage;
-  $update_fu = new Win32::API('shell32', 'SHChangeNotify', 'LIPP', 'V');
+  $update_fu = Win32::API::More->new('shell32', 'SHChangeNotify', 'LIPP', 'V');
   debug ("Import failure assoc_notify\n") unless $update_fu;
 }
 
@@ -197,9 +192,6 @@ my $windows_version = 0;
 my $windows_subversion = 0;
 
 if ($is_win) {
-  #my @osver = Win32::GetOSVersion(); # deprecated
-  #$windows_version = $osver[1];
-  #debug "Windows version $osver[0], major version $osver[1]\n";
   my $ver = `ver`;
   chomp $ver;
   $ver =~ s/^[^0-9]*//;
@@ -209,25 +201,6 @@ if ($is_win) {
   $windows_subversion =~ s/\..*$//;
 }
 
-=item C<is_vista>
-
-C<is_vista> returns 1 if windows version is >= 6.0, otherwise 0.
-
-=cut
-
-sub is_vista { return $windows_version >= 6; }
-
-=item C<is_seven>
-
-C<is_seven> returns 1 if windows version is >= 6.1, otherwise 0.
-
-=cut
-
-sub is_seven {
-  return (($windows_version == 6 && $windows_subversion >= 1) ||
-    ($windows_version > 6));
-}
-
 =item C<is_ten>
 
 C<is_ten> returns 1 if windows version is >= 10.0, otherwise 0.
@@ -271,7 +244,7 @@ to the system environment.
 
 =cut
 
-# during processing of this source, $is_admin will get its correct value
+# $is_admin has already got its correct value
 
 sub admin { return $is_admin; }
 
@@ -306,15 +279,9 @@ Two-letter country code representing the locale of the current user
 =cut
 
 sub reg_country {
-  my $value = Win32::OLE::NLS::GetUserDefaultLangID();
-  return unless $value;
-  $value = sprintf ("%04x", $value);
-  my $lmkey = $Registry -> Open("HKEY_CLASSES_ROOT/MIME/Database/Rfc1766/",
-                             {Access => KEY_READ()});
-  return unless $lmkey;
-  $lm = $lmkey->{"/$value"};
+  my $lm = cu_root()->{"Control Panel/international//localename"};
   return unless $lm;
-  debug("found lang codes value = $value, lm = $lm...\n");
+  debug("found lang code lm = $lm...\n");
   if ($lm) {
     if ($lm =~ m/^zh-(tw|hk)$/i) {
       return ("zh", "tw");
@@ -328,6 +295,7 @@ sub reg_country {
       return($lang, $area);
     }
   }
+  # otherwise undef will be returned
 }
 
 
@@ -377,7 +345,7 @@ sub is_a_texdir {
   $sr =~ s/\\/\//g;
   $sr = $sr . '/' unless $sr =~ m!/$!;
   return 0 if index($d, $sr)==0;
-  foreach $p (qw(luatex.exe mktexlsr.exe pdftex.exe tex.exe xetex.exe)) {
+  foreach my $p (qw(luatex.exe mktexlsr.exe pdftex.exe tex.exe xetex.exe)) {
     return 1 if (-e $d.$p);
   }
   return 0;
@@ -387,8 +355,7 @@ sub is_a_texdir {
 
 =item C<get_system_path>
 
-Returns unexpanded system path, as stored in the registry, but with
-forward slashes.
+Returns unexpanded system path, as stored in the registry.
 
 =cut
 
@@ -404,9 +371,8 @@ sub get_system_path {
 
 =item C<get_user_path>
 
-Returns unexpanded user path, as stored in the registry, but with
-forward slashes. The user path often does not exist, and is rarely
-expandable.
+Returns unexpanded user path, as stored in the registry. The user
+path often does not exist, and is rarely expandable.
 
 =cut
 
@@ -1032,15 +998,15 @@ sub unregister_file_type {
 =item C<broadcast_env>
 
 Broadcasts system message that enviroment has changed. This only has
-an effect on newly-started programs, not on running programs and the
+an effect on newly-started programs, not on running programs or the
 processes they spawn.
 
 =cut
 
 sub broadcast_env() {
   if ($SendMessage) {
-    use constant HWND_BROADCAST	=> 0xffff;
-    use constant WM_SETTINGCHANGE	=> 0x001A;
+    use constant HWND_BROADCAST => 0xffff;
+    use constant WM_SETTINGCHANGE => 0x001A;
     my $result = "";
     my $ans = "12345678"; # room for dword
     $result = $SendMessage->Call(HWND_BROADCAST, WM_SETTINGCHANGE,
@@ -1060,12 +1026,16 @@ Notifies the system that filetypes have changed.
 =cut
 
 sub update_assocs() {
-  use constant SHCNE_ASSOCCHANGED	=> 0x8000000;
-  use constant SHCNF_IDLIST =>	0;
+  use constant SHCNE_ASSOCCHANGED => 0x8000000;
+  use constant SHCNF_IDLIST => 0;
   if ($update_fu) {
     debug("Notifying changes in filetypes...\n");
-    $update_fu->Call (SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
-    debug("Done notifying\n");
+    my $result = $update_fu->Call(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, 0, 0);
+    if ($result) {
+      debug("Done notifying filetype changes\n");
+    } else{
+      debug("Failure notifying filetype changes\n");
+    }
   } else {
     debug("No update_fu\n");
   }
@@ -1233,7 +1203,8 @@ set PERL_ROOT=
 set PERL_SIGNALS=
 set PERL_UNICODE=
 
-perl.exe \"$tdmain\\scripts\\texlive\\uninstall-win32.pl\"
+perl.exe \"$tdmain\\scripts\\texlive\\uninstall-win32.pl\" \%1
+
 if errorlevel 1 goto :eof
 rem test for taskkill and try to stop exit tray menu
 taskkill /? >nul 2>&1
@@ -1349,6 +1320,12 @@ sub unregister_uninstaller {
 
 =pod
 
+=back
+
+=head2 ADMIN
+
+=over 4
+
 =item C<TeXLive::TLWinGoo::maybe_make_ro($dir)>
 
 Write-protects a directory $dir recursively, using ACLs, but only if
@@ -1356,6 +1333,8 @@ we are a multi-user install, and only if $dir is on an
 NTFS-formatted local fixed disk, and only on Windows Vista and
 later.  It writes a log message what it does and why.
 
+=back
+
 =cut
 
 sub maybe_make_ro {
@@ -1366,11 +1345,6 @@ sub maybe_make_ro {
     log "Not an admin install; not making read-only\n";
     return 1;
   }
-  if (!is_vista()) {
-    log "Windows version $windows_version earlier than Vista;".
-      " not making read-only\n";
-    return 1;
-  }
 
   $dir = Cwd::abs_path($dir);
 
@@ -1416,17 +1390,14 @@ sub maybe_make_ro {
   # S-1-5-11     Authenticated users
   # S-1-5-32-545 Users
   # S-1-5-32-544 administrators
-  # S-1-3-0      creator owner
+  # S-1-3-0      creator owner (does not work right)
   # S-1-3-1      creator group
 
-  # useful explanations at http://timbolton.net/2010/06/23/
-  #   /icacls-changing-permissions-on-files-and-folders
-
   # /reset is necessary for removing non-standard existing permissions
   my $cmd = 'cmd /c "icacls . /reset && icacls . /inheritance:r'.
-    ' /grant:r *S-1-5-32-544:(OI)(CI)F /grant *S-1-3-0:(OI)(CI)F'.
-    ' /grant *S-1-5-11:(OI)(CI)RX"';
-  log "Making read-only\n".`$cmd`."\n";
+    ' /grant:r *S-1-5-32-544:(OI)(CI)F'.
+    ' /grant:r *S-1-5-11:(OI)(CI)RX /grant:r *S-1-5-32-545:(OI)(CI)RX"';
+  log "Making read-only\n".Encode::decode(console_out,`$cmd`)."\n";
 
   # go back to original directory
   chdir $curdir;
